"""Exceptions used by alpm_srcinfo module."""

class SourceInfoError(Exception):
    """The high-level exception that can occur when using alpm_srcinfo module."""

__all__ = [
    "SourceInfoError",
]
